/**
  ****************************************************************************************************
  * @file    fm33fr0xx_fl_tsi.c
  * @author  FMSH Application Team
  * @brief   Src file of TSI FL Module
  ****************************************************************************************************
  * @attention
  *
  * Copyright (c) [2019] [Fudan Microelectronics]
  * THIS SOFTWARE is licensed under the Mulan PSL v1.
  * can use this software according to the terms and conditions of the Mulan PSL v1.
  * THIS SOFTWARE IS PROVIDED ON AN "AS IS" BASIS, WITHOUT WARRANTIES OF ANY KIND, EITHER EXPRESS OR
  * IMPLIED, INCLUDING BUT NOT LIMITED TO NON-INFRINGEMENT, MERCHANTABILITY OR FIT FOR A PARTICULAR
  * PURPOSE.
  * See the Mulan PSL v1 for more details.
  *
  ****************************************************************************************************
  */
/* Includes ------------------------------------------------------------------*/
#include "fm33fr0xx_fl.h"

/** @addtogroup FM33LC0XX_FL_Driver_TSI
  * @{
  */

/* Private macros ------------------------------------------------------------*/
/** @addtogroup TSI_FL_Private_Macros
  * @{
  */

#define GET_TSI_IO_PORT_ID(__PORT__)      (((GPIO_Type*)__PORT__) == GPIOA ? 0 : ( \
                                           ((GPIO_Type*)__PORT__) == GPIOB ? 1 : ( \
                                           ((GPIO_Type*)__PORT__) == GPIOC ? 2 : ( \
                                           ((GPIO_Type*)__PORT__) == GPIOD ? 3 : ( \
                                           ((GPIO_Type*)__PORT__) == GPIOE ? 4 : ( \
                                           ((GPIO_Type*)__PORT__) == GPIOF ? 5 : 0))))))

#define GET_TSI_IO_PIN_ID(__PIN__)        (((uint32_t)__PIN__) == FL_GPIO_PIN_0 ? 0 : ( \
                                           ((uint32_t)__PIN__) == FL_GPIO_PIN_1 ? 1 : ( \
                                           ((uint32_t)__PIN__) == FL_GPIO_PIN_2 ? 2 : ( \
                                           ((uint32_t)__PIN__) == FL_GPIO_PIN_3 ? 3 : ( \
                                           ((uint32_t)__PIN__) == FL_GPIO_PIN_4 ? 4 : ( \
                                           ((uint32_t)__PIN__) == FL_GPIO_PIN_5 ? 5 : ( \
                                           ((uint32_t)__PIN__) == FL_GPIO_PIN_6 ? 6 : ( \
                                           ((uint32_t)__PIN__) == FL_GPIO_PIN_7 ? 7 : ( \
                                           ((uint32_t)__PIN__) == FL_GPIO_PIN_8 ? 8 : ( \
                                           ((uint32_t)__PIN__) == FL_GPIO_PIN_9 ? 9 : ( \
                                           ((uint32_t)__PIN__) == FL_GPIO_PIN_10 ? 10 : ( \
                                           ((uint32_t)__PIN__) == FL_GPIO_PIN_11 ? 11 : ( \
                                           ((uint32_t)__PIN__) == FL_GPIO_PIN_12 ? 12 : ( \
                                           ((uint32_t)__PIN__) == FL_GPIO_PIN_13 ? 13 : ( \
                                           ((uint32_t)__PIN__) == FL_GPIO_PIN_14 ? 14 : ( \
                                           ((uint32_t)__PIN__) == FL_GPIO_PIN_15 ? 15 : 0))))))))))))))))

/*
    获取GPIO配置的唯一ID(16bit):
    - [15:10] 通道号
    - [9:4]   端口（GPIOA，B...）
    - [3:0]   引脚 (FL_GPIO_PIN_0 - FL_GPIO_PIN_15)
 */
#define GET_TSI_IO_UNIQUE_ID(__IO_INIT_STRUCT__)    ((uint16_t)((((FL_TSI_IOInitTypeDef*)(__IO_INIT_STRUCT__))->channel & 0x3FU) << 10U |    \
                                                               (GET_TSI_IO_PORT_ID(((FL_TSI_IOInitTypeDef*)(__IO_INIT_STRUCT__))->port) & 0x3FU) << 4U | \
                                                               (GET_TSI_IO_PIN_ID(((FL_TSI_IOInitTypeDef*)(__IO_INIT_STRUCT__))->pin) & 0xFU)))

/* 通过IO的UID判断是否为有效IO */
#define IS_TSI_VALID_IO(__VALUE__)                  (((__VALUE__) == (uint16_t)(0U  << 10U | 4U << 4U |  0U))|| /* 通道0  @ PE0  */   \
                                                     ((__VALUE__) == (uint16_t)(1U  << 10U | 4U << 4U |  1U))|| /* 通道1  @ PE1  */   \
                                                     ((__VALUE__) == (uint16_t)(2U  << 10U | 4U << 4U |  2U))|| /* 通道2  @ PE2  */   \
                                                     ((__VALUE__) == (uint16_t)(3U  << 10U | 4U << 4U |  3U))|| /* 通道3  @ PE3  */   \
                                                     ((__VALUE__) == (uint16_t)(4U  << 10U | 4U << 4U |  4U))|| /* 通道4  @ PE4  */   \
                                                     ((__VALUE__) == (uint16_t)(5U  << 10U | 2U << 4U |  3U))|| /* 通道5  @ PC3  */   \
                                                     ((__VALUE__) == (uint16_t)(6U  << 10U | 2U << 4U |  4U))|| /* 通道6  @ PC4  */   \
                                                     ((__VALUE__) == (uint16_t)(7U  << 10U | 3U << 4U |  7U))|| /* 通道7  @ PD7  */   \
                                                     ((__VALUE__) == (uint16_t)(8U  << 10U | 4U << 4U |  9U))|| /* 通道8  @ PE9  */   \
                                                     ((__VALUE__) == (uint16_t)(9U  << 10U | 4U << 4U | 10U))|| /* 通道9  @ PE10 */   \
                                                     ((__VALUE__) == (uint16_t)(10U << 10U | 0U << 4U | 10U))|| /* 通道10 @ PA10 */   \
                                                     ((__VALUE__) == (uint16_t)(11U << 10U | 0U << 4U | 11U))|| /* 通道11 @ PA11 */   \
                                                     ((__VALUE__) == (uint16_t)(12U << 10U | 0U << 4U | 12U)))  /* 通道12 @ PA12 */

/**
  * @}
  */

GPIO_Type *availablePorts[] =
{
    GPIOA, GPIOB, GPIOC, GPIOD, GPIOE,
    GPIOF, GPIOE,
};

/** @addtogroup TSI_FL_EF_InitavailablePorts
  * @{
  */

/**
 * @brief
 */
FL_ErrorStatus FL_TSI_ModuleReset(void)
{
    FL_RMU_EnablePeripheralReset(RMU);
    FL_RMU_EnableResetAPBPeripheral(RMU, FL_RMU_RSTAPB_TSC);
    FL_RMU_DisableResetAPBPeripheral(RMU, FL_RMU_RSTAPB_TSC);
    FL_RMU_DisablePeripheralReset(RMU);
    return FL_PASS;
}

/**
  * @brief  根据 initStruct 的配置信息初始化TSI的时钟
  * @param  clockSrc 时钟源选择
  * @param  clockPrescaler 时钟源分频选择
  * @retval ErrorStatus枚举值
  *         -FL_FAIL 配置过程发生错误
  *         -FL_PASS 配置成功
  */
FL_ErrorStatus FL_TSI_ClockInit(uint32_t clockSrc, uint32_t clockPrescaler, uint32_t clockFreq)
{
    uint32_t tmp;
    /* Unused parameters */
    (void) clockSrc;
    (void) clockPrescaler;
    /*
        FM33FR0xx的TSI模块固定使用APBCLK作为工作时钟，并且没有分频控制，只需使能
        总线时钟
    */
    FL_CMU_EnableGroup2BusClock(FL_CMU_GROUP2_BUSCLK_TSI);
    /* 检查设置的时钟频率是否符合实际结果 */
    tmp = FL_CMU_GetAPBClockFreq();
    if(tmp != clockFreq)
    {
        return FL_FAIL;
    }
    return FL_PASS;
}

/**
  * @brief  根据 initStruct 的配置信息初始化TSI的IO
  * @param  pinConfig 端口的配置信息，以唯一ID的形式表示
  * @retval ErrorStatus枚举值
  *         -FL_FAIL 配置过程发生错误
  *         -FL_PASS 配置成功
  */
FL_ErrorStatus FL_TSI_PinInit(uint32_t pinConfig)
{
    FL_GPIO_InitTypeDef gpio = {0};
    uint32_t portId = ((pinConfig & 0x3F0U) >> 4U);
    uint32_t pin = ((uint32_t)1U << (pinConfig & 0xFU));
    GPIO_Type *port;
    // 校验参数
    assert_param(IS_TSI_VALID_IO(pinConfig));
    assert_param(portId < (sizeof(availablePorts) / sizeof(GPIO_Type *)));
    // 获取端口指针
    port = availablePorts[portId];
    // 初始化IO
    FL_GPIO_StructInit(&gpio);
    gpio.pin = pin;
    gpio.mode = (uint32_t)FL_GPIO_MODE_ANALOG;
    (void)FL_GPIO_Init(port, &gpio);
    return FL_PASS;
}

/**
  * @brief  根据指定的优先级初始化TSI中断响应
  * @param  priority 优先级
  * @retval ErrorStatus枚举值
  *         -FL_FAIL 配置过程发生错误
  *         -FL_PASS 配置成功
  */
FL_ErrorStatus FL_TSI_InterruptInit(uint32_t priority)
{
    if(priority >= 3)
    {
        priority = 3;
    }
    NVIC_DisableIRQ(TSI_IRQn);
    NVIC_ClearPendingIRQ(TSI_IRQn);
    NVIC_SetPriority(TSI_IRQn, priority);
    NVIC_EnableIRQ(TSI_IRQn);
    return FL_PASS;
}

/**
  * @}
  */

/**
  * @}
  */
/*************************************************************END OF FILE************************************************************/
